package com.greatchn.collection.entity;

import com.greatchn.collection.vo.CcFileInfoVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 文件信息数据使用 Mapper 接口
 * </p>
 *
 * @author zhaolechang
 * @since 2022-08-20
 */
public interface CcFileInfoMapper {

    /**
     * 动态表 插入数据
     *
     * @param type
     * @param customSql
     */
    void addFileInfo(@Param("type") Integer type, @Param("customSql") String customSql);

    /**
     * 动态表 统计数量
     *
     * @param fileType
     * @param selectSql
     */
    long count(@Param("fileType") Integer fileType, @Param("selectSql") String selectSql);

    /**
     * 动态表 案卷统计数量
     *
     * @param fileType
     * @param selectSql
     */
    long arcCount(@Param("dh") String dh, @Param("fileType") Integer fileType, @Param("selectSql") String selectSql, @Param("ch") String ch);


    /**
     * 动态表 查询
     *
     * @param fileType
     * @param pageNum
     * @param pageSize
     * @return
     */
    List<Map> getList(@Param("fileType") Integer fileType, @Param("selectSql") String selectSql,
                      @Param("pageNum") Integer pageNum, @Param("pageSize") Integer pageSize);

    /**
     * 动态表 案卷查询
     *
     * @param fileType
     * @param pageNum
     * @param pageSize
     * @return
     */
    List<Map> getArcList(@Param("dh") String dh, @Param("fileType") Integer fileType, @Param("selectSql") String selectSql,
                         @Param("pageNum") Integer pageNum, @Param("pageSize") Integer pageSize, @Param("ch") String ch);

    List<Map> getFile(@Param("fileType") Integer fileType, @Param("selectSql") String selectSql);


    /**
     * 更新资料信息
     *
     * @return
     * @author zhaolechang
     * @creed: Talk is cheap,show me the code
     * @date 2022/8/19 15:27
     */
    int upDateFile(@Param("fileType") Integer fileType, @Param("customSql") String customSql, @Param("dzwjh") String dzwjh);

    /**
     * 更新案卷信息
     *
     * @return
     * @author zhaolechang
     * @creed: Talk is cheap,show me the code
     * @date 2022/8/22 8:42
     */
    int upDateArc(@Param("customSql") String customSql, @Param("dh") String dh);


    /**
     * 获取文件url
     *
     * @param fileTypeId
     * @param dzwjh
     * @return
     * @author zhaoding
     * @date 2022/8/22
     */
    Map getUrl(@Param("fileTypeId") Integer fileTypeId, @Param("dzwjh") String dzwjh);

    /**
     * 根据全宗名称、年、月与fileArr查询cc_file_info表
     *
     * @param fileType  类型id
     * @param selectSql 查询条件
     * @return
     */
    List<CcFileInfoVO> findCcFileInfo(@Param("fileType") Integer fileType, @Param("selectSql") String selectSql);

    /**
     * 根据全宗名称、时间范围查询cc_file_info表
     *
     * @param arr
     * @param M050
     * @param start
     * @param end
     * @return
     */
    List<CcFileInfoVO> findCcFileInfoByDate(@Param("arr") Integer arr, @Param("M050") String M050,
                                            @Param("start") String start, @Param("end") String end);

    /**
     * 根据arc_type、全宗名称与M066删除cc_file_info表中的数据
     *
     * @param arr     表名后缀
     * @param arcType 是否立卷
     * @param qzName  全宗名称 cc_file_info表中M005字段
     * @param m066
     */
    void delete(@Param("arr") Integer arr, @Param("arcType") String arcType, @Param("M005") String qzName, @Param("M066") String m066);

    /**
     * 获取要删除的主件电子文件号
     *
     * @param arr
     * @param arcType
     * @param qzName
     * @param m066
     * @return
     */
    String getDzwjh(@Param("arr") Integer arr, @Param("arcType") String arcType, @Param("M005") String qzName, @Param("M066") String m066);


    void deleteFj(@Param("arr") Integer arr, @Param("dzwjh") String dzwjh);

    /**
     * 查询未立卷文件数量
     *
     * @param fileTypeId 表名后缀
     * @return
     * @author zhangkun
     * @date 2023/1/12 13:57
     */
    long getNotLiJuanCount(@Param("fileTypeId") Integer fileTypeId);

    /**
     * 获取案卷内的文件信息
     * @param dh
     * @param fileType
     * @param textValue
     * @param groupSql
     * @return
     */
    List<Map> getFileList(@Param("dh") String dh, @Param("fileType") String fileType, @Param("textValue") List textValue,
                          @Param("groupSql") String groupSql);

    List<Map> getFileListOrder(@Param("dh") String dh, @Param("fileType") String fileType);
}
