package com.greatchn.collection.framework.interceptor;

import com.greatchn.collection.framework.constant.LogConstant;
import com.greatchn.collection.framework.constant.ReturnMsgEnum;
import com.greatchn.collection.framework.exception.AssertException;
import com.greatchn.collection.framework.vo.ReturnMsgVO;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.servlet.NoHandlerFoundException;

import java.util.Optional;

/**
 * 全局异常处理
 * Created by duhang on 2019/8/16 15:49
 */
@Slf4j
@ControllerAdvice(basePackages = {"com.greatchn"})
public class GlobalExceptionHandler {

    /**
     * 404
     * @param e
     * @return
     */
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    @ExceptionHandler(value = {NoHandlerFoundException.class})
    public ReturnMsgVO noHandlerFoundException(NoHandlerFoundException e) {
        log.warn("404", e);
        return ReturnMsgVO.newInstance().setCode(ReturnMsgEnum.NOT_FOUND.code()).setMsg(ReturnMsgEnum.NOT_FOUND.msg());
    }

    /**
     * 处理断言异常
     * @param e
     * @return
     */
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    @ExceptionHandler(value = AssertException.class)
    public ReturnMsgVO assertException(AssertException e) {
        log.warn("处理断言异常, ", e);
        return ReturnMsgVO.fail().setMsg(e.getMessage()).setCode(Optional.ofNullable(e.getCode()).orElse(ReturnMsgEnum.FAIL.code()));
    }

    /**
     * 处理服务器异常
     * @param e
     * @return
     */
    @ResponseStatus(HttpStatus.OK)
    @ResponseBody
    @ExceptionHandler(value = {Exception.class})
    public ReturnMsgVO unknownException(Exception e) {
        log.error("处理服务器异常, ", e);
        return ReturnMsgVO.error().setMsg("票盒服务发生异常，异常编号: " + MDC.get(LogConstant.TRACE_ID));
    }

}
