package com.greatchn.collection.framework.interceptor;

import com.greatchn.collection.framework.constant.LogConstant;
import com.greatchn.collection.framework.util.HttpUtil;
//import com.greatchn.orm.util.IdGenerator;
import eu.bitwalker.useragentutils.Browser;
import eu.bitwalker.useragentutils.OperatingSystem;
import eu.bitwalker.useragentutils.UserAgent;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;

/**
 * todo
 *
 * @author duhang
 * @date 2022/4/7
 */
@Component
public class LogInterceptor implements HandlerInterceptor {

    @Value("${app.name}")
    private String appName;
    @Value("${app.version}")
    private String appVersion;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {

        if ("options".equalsIgnoreCase(request.getMethod())) {
            return Boolean.TRUE;
        }

        Method method = ((HandlerMethod) handler).getMethod();
        String reqParam = HttpUtil.getRestMsg(request);
        reqParam = StringUtils.isNotBlank(reqParam) ? reqParam : HttpUtil.getBodyMsg(request);

//        MDC.put(LogConstant.TRACE_ID, IdGenerator.newID());
        MDC.put(LogConstant.SERVER_NAME, this.appName);
        MDC.put(LogConstant.SERVER_VERSION, this.appVersion);
        MDC.put(LogConstant.REQ_TYPE, "http:" + request.getMethod().toLowerCase());
        MDC.put(LogConstant.METHOD_NAME, method.getDeclaringClass().getSimpleName() + "." + method.getName());
        MDC.put(LogConstant.REQ_HEADER, HttpUtil.getReqHeader(request));
        MDC.put(LogConstant.REQ_URL, request.getRequestURL().toString());
        MDC.put(LogConstant.REQ_PARAM, reqParam);
        String clientIp = HttpUtil.getIpAddress(request);
        MDC.put(LogConstant.CLIENT_IP, clientIp);
        // 解析浏览器信息
        String userAgentStr = request.getHeader(HttpHeaders.USER_AGENT);
        if (StringUtils.isNotBlank(userAgentStr)) {
            UserAgent userAgent = UserAgent.parseUserAgentString(userAgentStr);
            Browser browser = userAgent.getBrowser();
            if (null != browser) {
                MDC.put(LogConstant.BROWSER_INFO, browser.getName() + ";" + browser.getBrowserType());
            }
            OperatingSystem system = userAgent.getOperatingSystem();
            if (null != system) {
                MDC.put(LogConstant.CLIENT_INFO, system.getName() + ";" + system.getDeviceType());
            }
        }
        return Boolean.TRUE;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

        if (!"options".equalsIgnoreCase(request.getMethod())) {
            MDC.clear();
        }
    }

}
