package com.greatchn.collection.framework.util;

import com.aspose.pdf.*;
import com.aspose.pdf.devices.PngDevice;
import com.aspose.pdf.devices.Resolution;
import com.aspose.pdf.facades.EncodingType;
import com.aspose.pdf.facades.FontStyle;
import com.aspose.pdf.facades.FormattedText;
import com.aspose.pdf.facades.PdfFileSignature;
import com.greatchn.collection.framework.constant.SignatureStateEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.DigestUtils;

import java.io.*;
import java.util.Date;

public class AposePdfUtil {
    public static boolean insertTextWatermark(String pdfFilePath, String watermarkText){
        boolean result = true;
        Document doc = new Document(pdfFilePath);

        // Create a formatted text
        FormattedText formattedText = new FormattedText(watermarkText, java.awt.Color.RED, FontStyle.Courier, EncodingType.Identity_h, true, 40.0F);

        // Create watermark artifact and set its properties
        WatermarkArtifact artifact = new WatermarkArtifact();
        artifact.setText(formattedText);
        artifact.setArtifactHorizontalAlignment (HorizontalAlignment.Center);
        artifact.setArtifactVerticalAlignment (VerticalAlignment.Center);
        artifact.setRotation (25);
        artifact.setOpacity (0.5);
        artifact.setBackground (false);

        for (int i = 1; i <= doc.getPages().size(); i++) {
            Page page = doc.getPages().get_Item(i);
            // Add watermark to the first page of PDF
            page.getArtifacts().add(artifact);
        }

        // Save watermarked PDF document
        doc.save("watermark.pdf");

        return result;
    }

    public static boolean insertImageWatermark(String pdfFilePath, String watermarkImagePath){
        boolean result = true;

        com.aspose.pdf.Document doc = new com.aspose.pdf.Document(pdfFilePath);

        // Create a background artifact
        BackgroundArtifact background = new BackgroundArtifact();
        try {
            // Specify the image for background artifact object
            background.setBackgroundImage(new FileInputStream(watermarkImagePath));
            background.setOpacity(0.5);
            background.setArtifactHorizontalAlignment(HorizontalAlignment.Center);
            background.setArtifactVerticalAlignment(VerticalAlignment.Center);

            for (int i = 1; i <= doc.getPages().size(); i++) {
                Page page = doc.getPages().get_Item(i);
                // Add watermark to the first page of PDF
                doc.getPages().get_Item(1).getArtifacts().add(background);
            }

            // Save watermarked PDF document
            doc.save(pdfFilePath);
        }catch (Exception ex){
            result = false;
            ex.printStackTrace();
        }

        return result;
    }

    public static boolean insertSeal(String pdfFilePath, String imagePath){
        boolean result = true;
        try {
            com.aspose.pdf.Document document = new com.aspose.pdf.Document(pdfFilePath);
            com.itextpdf.text.Image imgPhoto = com.itextpdf.text.Image.getInstance(imagePath);

            for (int i = 1; i <= document.getPages().size(); i++) {
                Page page = document.getPages().get_Item(i);
                double Width = document.getPageInfo().getWidth() / 4;
                double Height = document.getPageInfo().getHeight() / 4;
                com.aspose.pdf.ImageStamp imageStamp = new com.aspose.pdf.ImageStamp(imagePath);
                imageStamp.setQuality(100);
                imageStamp.setBackground(false);
                imageStamp.setXIndent(Width);
                imageStamp.setYIndent(Height);
                imageStamp.setHeight(imgPhoto.getHeight() * 0.5);
//                imageStamp.setWidth(50 * imgPhoto.getWidth() / imgPhoto.getHeight());
                imageStamp.setWidth(imgPhoto.getWidth() * 0.5);
                //    imageStamp.setRotate(Rotation.on270);
                imageStamp.setOpacity(0.7);
                page.addStamp(imageStamp);
            }
            document.save(pdfFilePath);
        }catch (Exception e){
            result = false;
        }
        return result;
    }

    public static String insertSignature(String pdfFilePath, String certFilePath, String password, String attribute, String imagePath){
        boolean result = true;

        try {
            com.itextpdf.text.Image imgPhoto = com.itextpdf.text.Image.getInstance(imagePath);
            String certPdfPath = pdfFilePath.replace(".pdf", "_cert.pdf");
            Document doc = new Document(pdfFilePath);
            PdfFileSignature signature = new PdfFileSignature(doc);
            PKCS7 pkcs = new PKCS7(certFilePath, password); // Use PKCS7/PKCS7Detached objects
            pkcs.setDate(new Date());
            DocMDPSignature docMdpSignature = new DocMDPSignature(pkcs, DocMDPAccessPermissions.FillingInForms);
            pkcs.setShowProperties(false);
            java.awt.Rectangle rect = new java.awt.Rectangle((int)(doc.getPageInfo().getWidth() - 100 ), 50,(int)(50 * imgPhoto.getWidth()/imgPhoto.getHeight()), 50 );
            // Set signature appearance
            signature.setSignatureAppearance(imagePath);
            // Create any of the three signature types
            //       signature.certify(1, "Signature Reason", "Contact", "Location", true, rect, docMdpSignature);
            signature.sign(1, "", DigestUtils.md5DigestAsHex((password + " " + attribute).getBytes()), "", false, rect, docMdpSignature.getSignature());
            // Save digitally signed PDF file
            signature.save(certPdfPath);
            signature.dispose();
            doc.dispose();

            return certPdfPath;
        }catch(Exception ex){
            ex.printStackTrace();
        }

        return pdfFilePath;
    }

    public static String analysisSignature(String pdfFilePath, String password, String attribute){
        password = DigestUtils.md5DigestAsHex((password + " " + attribute).getBytes());
        String result = SignatureStateEnum.NO_NAME.code();;

        // Create PDF File Signature
        PdfFileSignature pdfSign = new PdfFileSignature();
        // Bind PDF
        pdfSign.bindPdf(pdfFilePath);
        // Verify signature using signature name
        if (pdfSign.isCertified()){
            result = SignatureStateEnum.INVALID.code();

            if (pdfSign.verifySigned("Signature1")){
                if (pdfSign.getAccessPermissions() == DocMDPAccessPermissions.FillingInForms) { // Get access permission
                    String pass = pdfSign.getContactInfo("Signature1");
                    if(password.equals(pass)) {
                        result = SignatureStateEnum.VALID.code();
                    }
                }
            }
        }

        return result;
    }

    public static void pdfToPng(String pdfFilePath, String imageFilePath, Integer page) throws FileNotFoundException,IOException{
        // Open document
        Document pdfDocument = new Document(pdfFilePath);
        File file = new File(imageFilePath);
        // Create stream object to save the output image
        OutputStream imageStream = new FileOutputStream(imageFilePath);
        // Create Resolution object
        Resolution resolution = new Resolution(300);
        // Create PngDevice object with particular resolution
        PngDevice pngDevice = new PngDevice(resolution);
        // Convert a particular page and save the image to stream
        pngDevice.process(pdfDocument.getPages().get_Item(page), imageStream);
        // Close the stream
        imageStream.close();
    }
}
