package com.greatchn.collection.framework.util;

import com.greatchn.collection.framework.exception.AssertException;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.lang.Nullable;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.Map;
import java.util.function.Supplier;

/**
 * 断言工具
 *
 * @author duhang
 * @date 2022/4/4
 */
public final class AssertUtil {

    public static void state(boolean expression, String message) {
        if (!expression) {
            throw new AssertException(message);
        }
    }

    public static void state(boolean expression, String message, Integer code) {
        if (!expression) {
            throw new AssertException(code, message);
        }
    }

    public static void state(boolean expression, Supplier<String> messageSupplier) {
        if (!expression) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void state(boolean expression, Supplier<String> messageSupplier, Integer code) {
        if (!expression) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 expression 结果 为 true, 否则 抛出AssertException异常, 异常消息为message
     * @param expression
     * @param message
     */
    public static void isTrue(boolean expression, String message) {
        if (!expression) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 expression 结果 为 true, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param expression
     * @param message
     * @param code
     */
    public static void isTrue(boolean expression, String message, Integer code) {
        if (!expression) {
            throw new AssertException(code, message);
        }
    }

    public static void isTrue(boolean expression, Supplier<String> messageSupplier) {
        if (!expression) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void isTrue(boolean expression, Supplier<String> messageSupplier, Integer code) {
        if (!expression) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 expression 结果 为 false, 否则 抛出AssertException异常, 异常消息为message
     * @param expression
     * @param message
     */
    public static void isFalse(boolean expression, String message) {
        if (expression) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 expression 结果 为 false, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param expression
     * @param message
     * @param code
     */
    public static void isFalse(boolean expression, String message, Integer code) {
        if (expression) {
            throw new AssertException(code, message);
        }
    }

    /**
     * 判定 object 为 null, 否则 抛出AssertException异常, 异常消息为message
     * @param object
     * @param message
     */
    public static void isNull(@Nullable Object object, String message) {
        if (object != null) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 object 为 null, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param object
     * @param message
     * @param code
     */
    public static void isNull(@Nullable Object object, String message, Integer code) {
        if (object != null) {
            throw new AssertException(code, message);
        }
    }

    public static void isNull(@Nullable Object object, Supplier<String> messageSupplier) {
        if (object != null) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void isNull(@Nullable Object object, Supplier<String> messageSupplier, Integer code) {
        if (object != null) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 object 不为 null, 否则 抛出AssertException异常, 异常消息为message
     * @param object
     * @param message
     */
    public static void notNull(@Nullable Object object, String message) {
        if (object == null) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 object 不为 null, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param object
     * @param message
     * @param code
     */
    public static void notNull(@Nullable Object object, String message, Integer code) {
        if (object == null) {
            throw new AssertException(code, message);
        }
    }

    public static void notNull(@Nullable Object object, Supplier<String> messageSupplier) {
        if (object == null) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void notNull(@Nullable Object object, Supplier<String> messageSupplier, Integer code) {
        if (object == null) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    public static void hasLength(@Nullable String text, String message) {
        if (!StringUtils.hasLength(text)) {
            throw new AssertException(message);
        }
    }

    public static void hasLength(@Nullable String text, String message, Integer code) {
        if (!StringUtils.hasLength(text)) {
            throw new AssertException(code, message);
        }
    }

    public static void hasLength(@Nullable String text, Supplier<String> messageSupplier) {
        if (!StringUtils.hasLength(text)) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void hasLength(@Nullable String text, Supplier<String> messageSupplier, Integer code) {
        if (!StringUtils.hasLength(text)) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 text 不为null 且 不为空串, 否则 抛出AssertException异常, 异常消息为message
     * @param text
     * @param message
     */
    public static void hasText(@Nullable String text, String message) {
        if (!StringUtils.hasText(text)) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 text 不为null 且 不为空串, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param text
     * @param message
     * @param code
     */
    public static void hasText(@Nullable String text, String message, Integer code) {
        if (!StringUtils.hasText(text)) {
            throw new AssertException(code, message);
        }
    }

    public static void hasText(@Nullable String text, Supplier<String> messageSupplier) {
        if (!StringUtils.hasText(text)) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void hasText(@Nullable String text, Supplier<String> messageSupplier, Integer code) {
        if (!StringUtils.hasText(text)) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    public static void doesNotContain(@Nullable String textToSearch, String substring, String message) {
        if (StringUtils.hasLength(textToSearch) && StringUtils.hasLength(substring) && textToSearch.contains(substring)) {
            throw new AssertException(message);
        }
    }

    public static void doesNotContain(@Nullable String textToSearch, String substring, String message, Integer code) {
        if (StringUtils.hasLength(textToSearch) && StringUtils.hasLength(substring) && textToSearch.contains(substring)) {
            throw new AssertException(code, message);
        }
    }

    public static void doesNotContain(@Nullable String textToSearch, String substring, Supplier<String> messageSupplier) {
        if (StringUtils.hasLength(textToSearch) && StringUtils.hasLength(substring) && textToSearch.contains(substring)) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void doesNotContain(@Nullable String textToSearch, String substring, Supplier<String> messageSupplier, Integer code) {
        if (StringUtils.hasLength(textToSearch) && StringUtils.hasLength(substring) && textToSearch.contains(substring)) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 array 不为null 且 length > 0, 否则 抛出AssertException异常, 异常消息为message
     * @param array
     * @param message
     */
    public static void notEmpty(@Nullable Object[] array, String message) {
        if (ObjectUtils.isEmpty(array)) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 array 不为null 且 length > 0, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param array
     * @param message
     * @param code
     */
    public static void notEmpty(@Nullable Object[] array, String message, Integer code) {
        if (ObjectUtils.isEmpty(array)) {
            throw new AssertException(code, message);
        }
    }

    public static void notEmpty(@Nullable Object[] array, Supplier<String> messageSupplier) {
        if (ObjectUtils.isEmpty(array)) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void notEmpty(@Nullable Object[] array, Supplier<String> messageSupplier, Integer code) {
        if (ObjectUtils.isEmpty(array)) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 array 不为null 且 每个元素都不为null, 否则 抛出AssertException异常, 异常消息为message
     * @param array
     * @param message
     */
    public static void noNullElements(@Nullable Object[] array, String message) {
        if (array != null) {
            Object[] var2 = array;
            int var3 = array.length;

            for (int var4 = 0; var4 < var3; ++var4) {
                Object element = var2[var4];
                if (element == null) {
                    throw new AssertException(message);
                }
                if (element instanceof String && !StringUtils.hasText(String.valueOf(element))) {
                    throw new AssertException(message);
                }
            }
        }
    }

    /**
     * 判定 array 不为null 且 每个元素都不为null, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param array
     * @param message
     * @param code
     */
    public static void noNullElements(@Nullable Object[] array, String message, Integer code) {
        if (array != null) {
            Object[] var2 = array;
            int var3 = array.length;

            for (int var4 = 0; var4 < var3; ++var4) {
                Object element = var2[var4];
                if (element == null) {
                    throw new AssertException(code, message);
                }
                if (element instanceof String && !StringUtils.hasText(String.valueOf(element))) {
                    throw new AssertException(code, message);
                }
            }
        }
    }

    public static void noNullElements(@Nullable Object[] array, Supplier<String> messageSupplier) {
        if (array != null) {
            Object[] var2 = array;
            int var3 = array.length;

            for (int var4 = 0; var4 < var3; ++var4) {
                Object element = var2[var4];
                if (element == null) {
                    throw new AssertException(nullSafeGet(messageSupplier));
                }
                if (element instanceof String && !StringUtils.hasText(String.valueOf(element))) {
                    throw new AssertException(nullSafeGet(messageSupplier));
                }
            }
        }
    }

    public static void noNullElements(@Nullable Object[] array, Supplier<String> messageSupplier, Integer code) {
        if (array != null) {
            Object[] var2 = array;
            int var3 = array.length;

            for (int var4 = 0; var4 < var3; ++var4) {
                Object element = var2[var4];
                if (element == null) {
                    throw new AssertException(code, nullSafeGet(messageSupplier));
                }
                if (element instanceof String && !StringUtils.hasText(String.valueOf(element))) {
                    throw new AssertException(code, nullSafeGet(messageSupplier));
                }
            }
        }
    }

    /**
     * 判定 collection 为 空, 否则 抛出AssertException异常, 异常消息为message
     * @param collection
     * @param message
     */
    public static void isEmpty(@Nullable Collection<?> collection, String message) {
        if (CollectionUtils.isNotEmpty(collection)) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 collection 为 空, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param collection
     * @param message
     * @param code
     */
    public static void isEmpty(@Nullable Collection<?> collection, String message, Integer code) {
        if (CollectionUtils.isNotEmpty(collection)) {
            throw new AssertException(code, message);
        }
    }

    /**
     * 判定 collection 不为 空, 否则 抛出AssertException异常, 异常消息为message
     * @param collection
     * @param message
     */
    public static void notEmpty(@Nullable Collection<?> collection, String message) {
        if (CollectionUtils.isEmpty(collection)) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 collection 不为 空, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param collection
     * @param message
     * @param code
     */
    public static void notEmpty(@Nullable Collection<?> collection, String message, Integer code) {
        if (CollectionUtils.isEmpty(collection)) {
            throw new AssertException(code, message);
        }
    }

    public static void notEmpty(@Nullable Collection<?> collection, Supplier<String> messageSupplier) {
        if (CollectionUtils.isEmpty(collection)) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void notEmpty(@Nullable Collection<?> collection, Supplier<String> messageSupplier, Integer code) {
        if (CollectionUtils.isEmpty(collection)) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 map 不为 空, 否则 抛出AssertException异常, 异常消息为message
     * @param map
     * @param message
     */
    public static void notEmpty(@Nullable Map<?, ?> map, String message) {
        if (null == map || map.isEmpty()) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 map 不为 空, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param map
     * @param message
     * @param code
     */
    public static void notEmpty(@Nullable Map<?, ?> map, String message, Integer code) {
        if (null == map || map.isEmpty()) {
            throw new AssertException(code, message);
        }
    }

    public static void notEmpty(@Nullable Map<?, ?> map, Supplier<String> messageSupplier) {
        if (null == map || map.isEmpty()) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void notEmpty(@Nullable Map<?, ?> map, Supplier<String> messageSupplier, Integer code) {
        if (null == map || map.isEmpty()) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    /**
     * 判定 map 为 空, 否则 抛出AssertException异常, 异常消息为message
     * @param map
     * @param message
     */
    public static void isEmpty(@Nullable Map<?, ?> map, String message) {
        if (null != map && !map.isEmpty()) {
            throw new AssertException(message);
        }
    }

    /**
     * 判定 map 不为 空, 否则 抛出AssertException异常, 异常消息为message, 异常代码为code
     * @param map
     * @param message
     * @param code
     */
    public static void isEmpty(@Nullable Map<?, ?> map, String message, Integer code) {
        if (null != map && !map.isEmpty()) {
            throw new AssertException(code, message);
        }
    }

    public static void isEmpty(@Nullable Map<?, ?> map, Supplier<String> messageSupplier) {
        if (null != map && !map.isEmpty()) {
            throw new AssertException(nullSafeGet(messageSupplier));
        }
    }

    public static void isEmpty(@Nullable Map<?, ?> map, Supplier<String> messageSupplier, Integer code) {
        if (null != map && !map.isEmpty()) {
            throw new AssertException(code, nullSafeGet(messageSupplier));
        }
    }

    public static void isInstanceOf(Class<?> type, @Nullable Object obj, String message) {
        notNull(type, (String) "Type to check against must not be null");
        if (!type.isInstance(obj)) {
            instanceCheckFailed(type, obj, message);
        }
    }

    public static void isInstanceOf(Class<?> type, @Nullable Object obj, String message, Integer code) {
        notNull(type, (String) "Type to check against must not be null");
        if (!type.isInstance(obj)) {
            instanceCheckFailed(type, obj, message, code);
        }
    }

    public static void isInstanceOf(Class<?> type, @Nullable Object obj, Supplier<String> messageSupplier) {
        notNull(type, (String) "Type to check against must not be null");
        if (!type.isInstance(obj)) {
            instanceCheckFailed(type, obj, nullSafeGet(messageSupplier));
        }
    }

    public static void isInstanceOf(Class<?> type, @Nullable Object obj, Supplier<String> messageSupplier, Integer code) {
        notNull(type, (String) "Type to check against must not be null");
        if (!type.isInstance(obj)) {
            instanceCheckFailed(type, obj, nullSafeGet(messageSupplier), code);
        }
    }

    public static void isInstanceOf(Class<?> type, @Nullable Object obj) {
        isInstanceOf(type, obj, "");
    }

    public static void isInstanceOf(Class<?> type, @Nullable Object obj, Integer code) {
        isInstanceOf(type, obj, "", code);
    }

    public static void isAssignable(Class<?> superType, @Nullable Class<?> subType, String message) {
        notNull(superType, (String) "Super type to check against must not be null");
        if (subType == null || !superType.isAssignableFrom(subType)) {
            assignableCheckFailed(superType, subType, message);
        }
    }

    public static void isAssignable(Class<?> superType, @Nullable Class<?> subType, String message, Integer code) {
        notNull(superType, (String) "Super type to check against must not be null");
        if (subType == null || !superType.isAssignableFrom(subType)) {
            assignableCheckFailed(superType, subType, message, code);
        }
    }

    public static void isAssignable(Class<?> superType, @Nullable Class<?> subType, Supplier<String> messageSupplier) {
        notNull(superType, (String) "Super type to check against must not be null");
        if (subType == null || !superType.isAssignableFrom(subType)) {
            assignableCheckFailed(superType, subType, nullSafeGet(messageSupplier));
        }
    }

    public static void isAssignable(Class<?> superType, @Nullable Class<?> subType, Supplier<String> messageSupplier, Integer code) {
        notNull(superType, (String) "Super type to check against must not be null");
        if (subType == null || !superType.isAssignableFrom(subType)) {
            assignableCheckFailed(superType, subType, nullSafeGet(messageSupplier), code);
        }
    }

    public static void isAssignable(Class<?> superType, Class<?> subType) {
        isAssignable(superType, subType, "");
    }

    public static void isAssignable(Class<?> superType, Class<?> subType, Integer code) {
        isAssignable(superType, subType, "", code);
    }

    private static void instanceCheckFailed(Class<?> type, @Nullable Object obj, @Nullable String msg) {
        String className = obj != null ? obj.getClass().getName() : "null";
        String result = "";
        boolean defaultMessage = true;
        if (StringUtils.hasLength(msg)) {
            if (endsWithSeparator(msg)) {
                result = msg + " ";
            } else {
                result = messageWithTypeName(msg, className);
                defaultMessage = false;
            }
        }

        if (defaultMessage) {
            result = result + "Object of class [" + className + "] must be an instance of " + type;
        }

        throw new AssertException(result);
    }

    private static void instanceCheckFailed(Class<?> type, @Nullable Object obj, @Nullable String msg, @Nullable Integer code) {
        String className = obj != null ? obj.getClass().getName() : "null";
        String result = "";
        boolean defaultMessage = true;
        if (StringUtils.hasLength(msg)) {
            if (endsWithSeparator(msg)) {
                result = msg + " ";
            } else {
                result = messageWithTypeName(msg, className);
                defaultMessage = false;
            }
        }
        if (defaultMessage) {
            result = result + "Object of class [" + className + "] must be an instance of " + type;
        }
        throw new AssertException(code, result);
    }

    private static void assignableCheckFailed(Class<?> superType, @Nullable Class<?> subType, @Nullable String msg) {
        String result = "";
        boolean defaultMessage = true;
        if (StringUtils.hasLength(msg)) {
            if (endsWithSeparator(msg)) {
                result = msg + " ";
            } else {
                result = messageWithTypeName(msg, subType);
                defaultMessage = false;
            }
        }
        if (defaultMessage) {
            result = result + subType + " is not assignable to " + superType;
        }
        throw new AssertException(result);
    }

    private static void assignableCheckFailed(Class<?> superType, @Nullable Class<?> subType, @Nullable String msg, @Nullable Integer code) {
        String result = "";
        boolean defaultMessage = true;
        if (StringUtils.hasLength(msg)) {
            if (endsWithSeparator(msg)) {
                result = msg + " ";
            } else {
                result = messageWithTypeName(msg, subType);
                defaultMessage = false;
            }
        }
        if (defaultMessage) {
            result = result + subType + " is not assignable to " + superType;
        }
        throw new AssertException(code, result);
    }

    private static boolean endsWithSeparator(String msg) {
        return msg.endsWith(":") || msg.endsWith(";") || msg.endsWith(",") || msg.endsWith(".");
    }

    private static String messageWithTypeName(String msg, @Nullable Object typeName) {
        return msg + (msg.endsWith(" ") ? "" : ": ") + typeName;
    }

    @Nullable
    private static String nullSafeGet(@Nullable Supplier<String> messageSupplier) {
        return messageSupplier != null ? messageSupplier.get() : null;
    }
}
