package com.greatchn.collection.framework.util;


import org.apache.commons.lang3.StringUtils;

/**
 * @description 权限信息工具类
 * @author duhang
 * @date 2021/7/11
 */
public final class AuthUtil {


    private static ThreadLocal<Long> S_EMPLOYEE_ID = new ThreadLocal<>();

    private static ThreadLocal<Long> S_CUSTOMER_ID = new ThreadLocal<>();

    private static ThreadLocal<String> S_ACCOUNT_ID = new ThreadLocal<>();

    private AuthUtil() {}

    /**
     * 当前线程存入员工id
     * @param employeeId
     */
    public static void setEmployeeId(Long employeeId) {
        if (null != employeeId) {
            S_EMPLOYEE_ID.set(employeeId);
        }
    }

    /**
     * 当前线程获取员工id
     * @return
     */
    public static Long getEmployeeId() {
        return S_EMPLOYEE_ID.get();
    }

    /**
     * 当前线程存入企业id
     * @param customerId
     */
    public static void setCustomerId(Long customerId) {
        if (null != customerId) {
            S_CUSTOMER_ID.set(customerId);
        }
    }

    /**
     * 当前线程获取企业id
     * @return
     */
    public static Long getCustomerId() {
        return S_CUSTOMER_ID.get();
    }

    /**
     * 当前线程存入账套id
     * @param accountId
     */
    public static void setAccountId(String accountId) {
        if (StringUtils.isNotBlank(accountId)) {
            S_ACCOUNT_ID.set(accountId);
        }
    }

    /**
     * 当前线程获取账套id
     * @return
     */
    public static String getAccountId() {
        return S_ACCOUNT_ID.get();
    }

    /**
     * 清空当前线程
     */
    public static void clear() {
        S_CUSTOMER_ID.remove();
        S_EMPLOYEE_ID.remove();
        S_ACCOUNT_ID.remove();
    }
}
