package com.greatchn.collection.framework.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 * @author duhang
 * @description hutool DateUtil扩展
 * @date 2021/10/19
 */
public final class DateUtil extends cn.hutool.core.date.DateUtil {

    public static final String YYYY = "yyyy";

    public static final String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static final String YYYYMM = "yyyyMM";

    public static final String YYYYMMDD = "yyyyMMdd";

    public static final String YYYYMMDD_HH_MM_SS = "yyyyMMdd HH:mm:ss";

    public static final String YYMMDDHHMMSS = "yyMMddHHmmss";

    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    public static final String YYYY_MM_DD = "yyyy-MM-dd";

    public static final String YYYY_MM_DD_STR = "yyyy年MM月dd日";

    public static final String HH_MM_SS = "HH:mm:ss";


    public static final String ZONE = "Asia/Shanghai";

    public static ZoneId ZONE_ID = ZoneId.of(ZONE);

    private DateUtil() {}

    /**
     * 获取上海时间
     * @return
     */
    public static LocalDateTime getCurrentShanghai() {

        return LocalDateTime.now(ZONE_ID);
    }

    /**
     * 获取北京时间
     *
     * @return
     */
    public static Date getCurrentBeijing() {

        Calendar calendar = Calendar.getInstance(Locale.CHINA);

        return calendar.getTime();
    }

    /**
     * 校验日期是否满足一定格式
     *
     * @author: ZBoHang
     * @time: 2021/10/22 15:44
     */
    public static boolean checkDateFormat(String dateStr, String formatStr) {
        SimpleDateFormat dateFormat = new SimpleDateFormat(formatStr);
        try {
            dateFormat.parse(dateStr);
        } catch (ParseException e) {
            return Boolean.FALSE;
        }
        return Boolean.TRUE;
    }

    /**
     * 返回这个日期前一个月的日期
     *
     * @param dateStr      日期字符串
     * @param sourceFormat 原日期格式
     * @param targetFormat 目标日期格式
     * @return: java.util.Date
     * @author: ZBoHang
     * @time: 2021/11/15 15:26
     */
    public static String getOneMonthBefore(String dateStr, String sourceFormat, String targetFormat) {

        // 一个月(30天)的毫秒数
        long oneDayMs = 1000 * 60 * 60 * 24 * 30L;

        long sourceTimes = DateUtil.parse(dateStr, sourceFormat).getTime();

        Date date = new Date(sourceTimes - oneDayMs);

        return DateUtil.format(date, targetFormat);
    }

	/**
     * 获取当前时间
     *
     */
    public static String getNow() {
        SimpleDateFormat formatter = new SimpleDateFormat(YYYY_MM_DD_STR);
        String str = formatter.format(new Date());
        return str;
    }

    /**
     * 将Date类型时间转换是指定日期格式
     * @param date
     * @param format
     */
    public static String formatDate(Date date, String format){
        SimpleDateFormat formatter = new SimpleDateFormat(format);
        String str = formatter.format(date);
        return str;
    }

    /**
     * 将String类型时间转换格式Date类型时间
     * @param dateStr
     * @param format
     */
    public static Date StrToDate(String dateStr, String format) throws Exception{
        SimpleDateFormat formatter = new SimpleDateFormat(format);
        Date date = new Date();
        try {
            date = formatter.parse(dateStr);
        } catch (ParseException e) {
           throw new Exception("时间字符串转换Date类型报错");
        }
        return date;
    }
}
