package com.greatchn.collection.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.greatchn.collection.entity.FilePO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.greatchn.collection.entity.FileMapper;
import com.greatchn.collection.framework.constant.ManageConstant;
import com.greatchn.collection.framework.exception.AssertException;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.commons.codec.digest.DigestUtils;

import java.io.*;
import java.util.List;

@Service
public class FileService  {
    @Autowired
    private FileMapper fileMapper;

    public FilePO save(InputStream inputStream, String targetFilePath, String targetFileName){
        // 获取文件的扩展名
        String ext = FilenameUtils.getExtension(targetFileName);
        inputStream.mark(0);
        String md5 = "";
        OutputStream os = null;
        File outFile = new File(targetFilePath + File.separator + targetFileName + ".temp");

        try {
            md5 = DigestUtils.md5Hex(inputStream);
        } catch (IOException e) {
            e.printStackTrace();
        }
        FilePO vo = this.check(targetFileName, md5);

        // 1K的数据缓冲
        byte[] bs = new byte[1024];
        // 读取到的数据长度
        int len;

        if(vo == null) {
            //数据库持久化这个数据
            FilePO file1 = new FilePO();
            file1.setPath(targetFilePath + File.separator + targetFileName);
            file1.setName(targetFileName);
            file1.setSuffix(ext);
            file1.setFileKey(md5);
            file1.setState(ManageConstant.CommonStateEnum.COLIN.getCode());

            this.save(file1);

            try {
                // 2、保存到临时文件
                // 输出的文件流保存到本地文件
                File tempFile = new File(targetFilePath);
                if (!tempFile.exists()) {
                    tempFile.mkdirs();
                }

                os = new FileOutputStream(outFile);
                inputStream.reset();
                int i = 0;

                // 开始读取
                while ((len = inputStream.read(bs)) != -1) {
                    os.write(bs, 0, len);
                }
            } catch (IOException e) {
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                // 完毕，关闭所有链接
                try {
                    os.close();
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                outFile.renameTo(new File(targetFilePath + File.separator + targetFileName));
                file1.setState(ManageConstant.CommonStateEnum.COLEND.getCode());
                this.save(file1);
            }
            return file1;
        }else{
            if(!ManageConstant.CommonStateEnum.COLEND.getCode().equals(vo.getState())) {

                try {
                    inputStream.reset();
                    int position = (int) outFile.length();
                    os = new FileOutputStream(outFile, true);

                    BufferedInputStream bis = new BufferedInputStream(inputStream);
                    bis.skip(position);

                    if (bis.markSupported()) {
//                    bis.reset();
                        // 开始读取
                        while ((len = bis.read(bs)) != -1) {
                            os.write(bs, 0, len);
                        }
                    } else {
                        System.out.print("InputStream不支持reset()");
                    }
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                } finally {
                    // 完毕，关闭所有链接
                    try {
                        os.close();
                        inputStream.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }

                    outFile.renameTo(new File(targetFileName));
                    vo.setState(ManageConstant.CommonStateEnum.COLEND.getCode());
                    this.save(vo);
                }
            }
            return vo;
        }
    }

    //保存文件
    public void save(FilePO file1){
        //根据 数据库的 文件标识来查询 当前文件 是否存在
        LambdaQueryWrapper<FilePO> lambda = new QueryWrapper<FilePO>().lambda();
        lambda.eq(FilePO::getFileKey,file1.getFileKey());
        List<FilePO> fileDTOS = fileMapper.selectList(lambda);

        //如果存在就修改
        if(fileDTOS.size()!=0){
            //根据key来修改
            LambdaQueryWrapper<FilePO> lambda1 = new QueryWrapper<FilePO>().lambda();
            lambda1.eq(FilePO::getFileKey,file1.getFileKey());
            fileMapper.update(file1,lambda1);
        }else{
            //不存在就添加
            fileMapper.insert(file1);
        }
    }

    //检查文件
    public FilePO check(String name, String key) {
        LambdaQueryWrapper<FilePO> lambda = new QueryWrapper<FilePO>().lambda();
        lambda.eq(FilePO::getName, name);
        FilePO dto = fileMapper.selectOne(lambda);

        if(dto!=null && !key.equals(dto.getFileKey())){
            fileMapper.deleteById(dto);
            dto = null;
        }
        return dto;
    }

    //检查文件
    public List<FilePO> check(String key) {
        LambdaQueryWrapper<FilePO> lambda = new QueryWrapper<FilePO>().lambda();
        lambda.eq(FilePO::getFileKey, key);
        List<FilePO> dtos = fileMapper.selectList(lambda);
        return dtos;
    }

}
